/***************************************************************************//**
* \file main_cm4.c
*
* \version 1.0
*
* \brief Main example file for CM4
*
********************************************************************************
* \copyright
* Copyright 2016-2020, Cypress Semiconductor Corporation. All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/
#include <stdio.h>
#include "cy_project.h"
#include "cy_device_headers.h"

#define BB_POTI_ANALOG_MACRO        PASS0_SAR0
#define BB_POTI_ANALOG_PCLK         CY_ADC_POT_PCLK
#define BB_POTI_ANALOG_INPUT_NO    ((cy_en_adc_pin_address_t)CY_ADC_POT_IN_NO)

#define DIV_ROUND_UP(a,b) (((a) + (b)/2) / (b))

/* A/D value result buff */
uint8_t         resultIdx;
uint16_t        resultBuff[16];
/* A/D Status */
cy_stc_adc_ch_status_t statusBuff[16];

/* PWM CONFIGURATION */
/* PWM Mode Configuration def */
#define TCPWMx_GRPx_CNTx_PWM          TCPWM0_GRP1_CNT0
#define PCLK_TCPWMx_CLOCKSx_PWM       PCLK_TCPWM0_CLOCKS256
#define TCPWM_PERI_CLK_DIVIDER_NO_PWM 0ul
#define TCPWMx_PWM_PRESCALAR_DIV_x    CY_TCPWM_PRESCALER_DIVBY_128 // 2,000,000 / 128 = 15,625Hz
#define TCPWMx_PERIOD                 0x1000 // 15,625Hz / 4096 (0x1000) = 3.815Hz (PWM frequency)
#define TCPWMx_COMPARE0               0x800 // 0x800 / 0x1000 = 0.5 (PWM duty)

/* TCPWM_LINE0 */
#define TCPWMx_LINEx_PORT             GPIO_PRT3
#define TCPWMx_LINEx_PIN              5
#define TCPWMx_LINEx_MUX              P3_5_TCPWM0_LINE256

// Definition of SAR ADC AC Specified Parameters
#define ADC_OPERATION_FREQUENCY_MAX_IN_HZ (26670000ul)
#define ANALOG_IN_SAMPLING_TIME_MIN_IN_NS (412ull)

#define ADC_LOGICAL_CHANNEL         0
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/


/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/

/* PWM */
cy_stc_gpio_pin_config_t pin_cfg1 =
{
  .outVal     = 0ul,
  .driveMode  = CY_GPIO_DM_STRONG_IN_OFF,
  .hsiom      = TCPWMx_LINEx_MUX,
  .intEdge    = 0ul,
  .intMask    = 0ul,
  .vtrip      = 0ul,
  .slewRate   = 0ul,
  .driveSel   = 0ul,
};

cy_stc_tcpwm_pwm_config_t const MyPWM_config =
{
  .pwmMode          = CY_TCPWM_PWM_MODE_PWM,
  .clockPrescaler   = TCPWMx_PWM_PRESCALAR_DIV_x,
  .debug_pause      = false,
  .cc0MatchMode     = CY_TCPWM_PWM_TR_CTRL2_CLEAR,
  .overflowMode     = CY_TCPWM_PWM_TR_CTRL2_SET,
  .underflowMode    = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
  .cc1MatchMode     = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
  .deadTime         = 0ul,
  .deadTimeComp     = 0ul,
  .runMode          = CY_TCPWM_PWM_CONTINUOUS,
  .period           = TCPWMx_PERIOD - 1ul,
  .period_buff      = 0ul,
  .enablePeriodSwap = false,
  .compare0         = TCPWMx_COMPARE0,
  .compare1         = 0ul,
  .enableCompare0Swap = false,
  .enableCompare1Swap = false,
  .interruptSources = 0ul,
  .invertPWMOut     = 0ul,
  .invertPWMOutN    = 0ul,
  .killMode         = CY_TCPWM_PWM_STOP_ON_KILL,
  .switchInputMode  = 3ul,
  .switchInput      = 0ul,
  .reloadInputMode  = 3ul,
  .reloadInput      = 0ul,
  .startInputMode   = 3ul,
  .startInput       = 0ul,
  .kill0InputMode   = 3ul,
  .kill0Input       = 0ul,
  .kill1InputMode   = 3ul,
  .kill1Input       = 0ul,
  .countInputMode   = 3ul,
  .countInput       = 1ul,
};

/* ADC CONFIGURATION */
/* ADC port setting (Note default port setting after reset is just fine) */
cy_stc_gpio_pin_config_t adcPinConfig =
{
  .outVal     = 0ul,
  .driveMode  = CY_GPIO_DM_ANALOG,
  .hsiom      = P6_0_GPIO,
  .intEdge    = 0ul,
  .intMask    = 0ul,
  .vtrip      = 0ul,
  .slewRate   = 0ul,
  .driveSel   = 0ul,
};

/* ADC Channel Configuration */
cy_stc_adc_config_t adcConfig =
{
  .preconditionTime     = 0ul,
  .powerupTime          = 0ul,
  .enableIdlePowerDown  = false,
  .msbStretchMode       = CY_ADC_MSB_STRETCH_MODE_1CYCLE,
  .enableHalfLsbConv    = 0ul,
  .sarMuxEnable         = true,
  .adcEnable            = true,
  .sarIpEnable          = true,
};

cy_stc_adc_channel_config_t adcChannelConfig =
{
  /* Trigger type: Trigger from corresponding TCPWM channel */
  .triggerSelection         = CY_ADC_TRIGGER_TCPWM,
  .channelPriority          = 0ul,
  .preenptionType           = CY_ADC_PREEMPTION_FINISH_RESUME,
  .isGroupEnd               = true,
  /* CY_ADC_DONE_LEVEL_PULSE = 0 */
  .doneLevel                = CY_ADC_DONE_LEVEL_PULSE,
  .pinAddress               = BB_POTI_ANALOG_INPUT_NO,
  .portAddress              = CY_ADC_PORT_ADDRESS_SARMUX0,
  .extMuxSelect             = 0ul,
  .extMuxEnable             = true,
  .preconditionMode         = CY_ADC_PRECONDITION_MODE_OFF,
  .overlapDiagMode          = CY_ADC_OVERLAP_DIAG_MODE_OFF,
  .sampleTime               = 60ul,
  .calibrationValueSelect   = CY_ADC_CALIBRATION_VALUE_REGULAR,
  .postProcessingMode       = CY_ADC_POST_PROCESSING_MODE_NONE,
  .resultAlignment          = CY_ADC_RESULT_ALIGNMENT_RIGHT,
  .signExtention            = CY_ADC_SIGN_EXTENTION_UNSIGNED,
  .averageCount             = 0ul,
  .rightShift               = 0ul,
  .rangeDetectionMode       = CY_ADC_RANGE_DETECTION_MODE_INSIDE_RANGE,
  .rangeDetectionLoThreshold = 0x0000ul,
  .rangeDetectionHiThreshold = 0x0FFFul,
  .mask.grpDone             = true,
  .mask.grpCancelled        = false,
  .mask.grpOverflow         = false,
  .mask.chRange             = false,
  .mask.chPulse             = false,
  .mask.chOverflow          = false,
};

void AdcIntHandler(void)
{
  cy_stc_adc_interrupt_source_t intrSource = { false };
  /* Get interrupt source */
  Cy_Adc_Channel_GetInterruptMaskedStatus(&BB_POTI_ANALOG_MACRO->CH[ADC_LOGICAL_CHANNEL], &intrSource);

  // if(intrSource.grpDone)
  {
    /* Get the result(s) */
    Cy_Adc_Channel_GetResult(&BB_POTI_ANALOG_MACRO->CH[ADC_LOGICAL_CHANNEL], &resultBuff[resultIdx], &statusBuff[resultIdx]);

    /* Display ADC result */
    printf("\rADC result = %04d ", resultBuff[resultIdx]);
    
    /* Increment result idx */
    resultIdx = (resultIdx + 1) % (sizeof(resultBuff) / sizeof(resultBuff[0]));
    /* Clear interrupt source */
    Cy_Adc_Channel_ClearInterruptStatus(&BB_POTI_ANALOG_MACRO->CH[ADC_LOGICAL_CHANNEL], &intrSource);
  }
  // else
  // {
  //   /* Unexpected interrupt */
  //   CY_ASSERT(false);
  // }
}

/* This is an ADC example file for HW trigger: TCPWM */
int main(void)
{
  SystemInit();
  __enable_irq(); /* Enable global interrupts. */

  Cy_Semihosting_InitAll(CY_USB_SCB_TYPE, 115200ul, NULL, false);
  /* Initialize ADC */
  // uint32_t periFreq = 80000000ul;
  // uint32_t divNum = DIV_ROUND_UP(periFreq, ADC_OPERATION_FREQUENCY_MAX_IN_HZ);
  // uint32_t actualAdcOperationFreq = periFreq / divNum;

  // uint32_t samplingCycle = (uint32_t)DIV_ROUND_UP((ANALOG_IN_SAMPLING_TIME_MIN_IN_NS * (uint64_t)actualAdcOperationFreq), 1000000000ull);
  // adcChannelConfig.sampleTime = samplingCycle;

  // Cy_GPIO_Pin_Init(CY_ADC_POT_PORT, CY_ADC_POT_PIN, &adcPinConfig);

  /* ADC clock divider setting */
  {
      Cy_SysClk_PeriphAssignDivider(BB_POTI_ANALOG_PCLK, CY_SYSCLK_DIV_16_BIT, 0u);
      Cy_SysClk_PeriphSetDivider(CY_SYSCLK_DIV_16_BIT, 0u, 5u);
      Cy_SysClk_PeriphEnableDivider(CY_SYSCLK_DIV_16_BIT, 0u);
  }

  /*  Enable clk_hf1   */
  // Cy_SysClk_ClkPathSetSource(2u, CY_SYSCLK_CLKPATH_IN_IMO);
  // Cy_SysClk_HfClkEnable(CY_SYSCLK_HFCLK_1);
  // Cy_SysClk_HfClockSetSource(CY_SYSCLK_HFCLK_1, CY_SYSCLK_HFCLK_IN_CLKPATH2);

  // Cy_Adc_SetGenericTriggerInput(PASS0_EPASS_MMIO, ADC_LOGICAL_CHANNEL, 0u, 0u);
  Cy_Adc_Init(BB_POTI_ANALOG_MACRO, &adcConfig);
  Cy_Adc_Channel_Init(&BB_POTI_ANALOG_MACRO->CH[ADC_LOGICAL_CHANNEL], &adcChannelConfig);

  /*--------------------------------*/
  /* Clock Configuration for TCPWMs */
  /*--------------------------------*/
  /* Assign a programmable divider  for TCPWM0_GRPx_CNTx_PWM_PR */
  Cy_SysClk_PeriphAssignDivider(PCLK_TCPWMx_CLOCKSx_PWM, (cy_en_divider_types_t)CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM);
  /* Sets the 16-bit divider */
  Cy_SysClk_PeriphSetDivider((cy_en_divider_types_t)CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM, 39u); // Divider 39 --> 80MHz / (39+1) = 2MHz
  Cy_SysClk_PeriphEnableDivider((cy_en_divider_types_t)CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM);

  /* Register ADC interrupt handler and enable interrupt */
  cy_stc_sysint_irq_t irq_cfg;
  irq_cfg = (cy_stc_sysint_irq_t){
    .sysIntSrc = pass_0_interrupts_sar_0_IRQn, //pass_0_interrupts_sar_0_IRQn,
    .intIdx = CPUIntIdx3_IRQn,
    .isEnabled = true,
  };
  Cy_SysInt_InitIRQ(&irq_cfg);
  Cy_SysInt_SetSystemIrqVector(irq_cfg.sysIntSrc, AdcIntHandler);
  NVIC_SetPriority(irq_cfg.intIdx, 0ul);
  NVIC_EnableIRQ(irq_cfg.intIdx);

  /* Port Configuration for TCPWM */
  Cy_GPIO_Pin_Init(TCPWMx_LINEx_PORT, TCPWMx_LINEx_PIN, &pin_cfg1);
  /* Initialize TCPWM0_GRPx_CNTx_PWM_PR as PWM Mode & Enable */
  Cy_Tcpwm_Pwm_Init(TCPWMx_GRPx_CNTx_PWM, &MyPWM_config);

  /* Enable ADC ch. and PWM */
  Cy_Adc_Channel_Enable(&BB_POTI_ANALOG_MACRO->CH[ADC_LOGICAL_CHANNEL]);
  Cy_Tcpwm_Pwm_Enable(TCPWMx_GRPx_CNTx_PWM);
  Cy_Tcpwm_TriggerStart(TCPWMx_GRPx_CNTx_PWM);
  
  /* Trigger MUX */
  Cy_TrigMux_Connect1To1(TRIG_IN_1TO1_1_TCPWM_TO_PASS_CH_TR0,
                        0ul,
                        TRIGGER_TYPE_PASS_TR_SAR_CH_IN__EDGE,
                        0ul);
// Cy_TrigMux_SwTrigger(TRIG_IN_1TO1_1_TCPWM_TO_PASS_CH_TR0, TRIGGER_TYPE_EDGE, 1 /*output*/); 
  // Cy_Adc_Channel_SoftwareTrigger(&BB_POTI_ANALOG_MACRO->CH[ADC_LOGICAL_CHANNEL]);
  
  for(;;);
}
